/* -*-c++-*- source - Open Producer - Copyright (C) 2002 Don Burns
 * Distributed under the terms of the GNU LIBRARY GENERAL PUBLIC LICENSE (LGPL)
 * as published by the Free Software Foundation.
 */
#include <stdio.h>
#include <Producer/Thread>

using namespace Producer;

Thread::Thread() 
{
   _runon = -1;
   _thread_running = false;
   pthread_attr_init( &_attr );
}

Thread::~Thread()
{
#ifndef WIN32
    if( _thread_running )
    {
        cancel();
        // UNIX hangs here. ???
        //join();
    }
#endif
}

size_t Thread::getStackSize()
{
    size_t size;
    pthread_attr_getstacksize( &_attr, &size );

    return size;
}

pthread_t Thread::getThreadID() { return _tid; }

void Thread::setStackSize( size_t size )
{
    pthread_attr_setstacksize( &_attr, size );
}

void Thread::setProcessorAffinity( int cpunum )
{
#ifdef __sgi
    pthread_attr_setscope( &_attr, PTHREAD_SCOPE_BOUND_NP );
    _runon = cpunum;
#endif
}

void Thread::startThread()
{
    _thread_running = true;
    pthread_create( &_tid, 0, s_StartThread, static_cast<void *>(this));  
}

void Thread::join()
{
     pthread_join( _tid, 0 );
}

void Thread::cancel()
{
    pthread_cancel( _tid );
}

void *Thread::s_StartThread(void *data)
{
    Thread *thread = static_cast<Thread *>(data);
#ifdef __sgi
    if( thread->_runon >= 0 )
        pthread_setrunon_np( thread->_runon );
    int th_policy;
    sched_param th_param;
    pthread_getschedparam(pthread_self(),
                          &th_policy, &th_param);
    pthread_setschedparam(pthread_self(),
                          SCHED_RR,
                          &th_param);

#endif
    pthread_setcancelstate( PTHREAD_CANCEL_ENABLE, 0L );
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, 0L );
    return thread->run();
}
